@@ -23,6 +23,7 @@
 
 NSRect PRectangleToNSRect(Scintilla::PRectangle& rc);
 Scintilla::PRectangle NSRectToPRectangle(NSRect& rc);
+CFStringEncoding EncodingFromCharacterSet(bool unicode, int characterSet);
 
 @interface ScintillaContextMenu : NSMenu
 {
@@ -47,14 +48,17 @@ private:
 
   /** The text layout instance */
   QuartzTextLayout*	textLayout;
+  int codePage;
+  int verticalDeviceResolution;
+	
   /** If the surface is a bitmap context, contains a reference to the bitmap data. */
   uint8_t* bitmapData;
   /** If the surface is a bitmap context, stores the dimensions of the bitmap. */
   int bitmapWidth;
   int bitmapHeight;
 
-  /** Set the CGContext's fill colour to the specified allocated colour. */
-  void FillColour( const ColourAllocated& back );
+  /** Set the CGContext's fill colour to the specified desired colour. */
+  void FillColour( const ColourDesired& back );
 
 
   // 24-bit RGB+A bitmap data constants
@@ -72,7 +76,7 @@ public:
 
   void Release();
   bool Initialised();
-  void PenColour(ColourAllocated fore);
+  void PenColour(ColourDesired fore);
 
   /** Returns a CGImageRef that represents the surface. Returns NULL if this is not possible. */
   CGImageRef GetImage();
@@ -82,36 +86,36 @@ public:
   int DeviceHeightFont(int points);
   void MoveTo(int x_, int y_);
   void LineTo(int x_, int y_);
-  void Polygon(Scintilla::Point *pts, int npts, ColourAllocated fore, ColourAllocated back);
-  void RectangleDraw(PRectangle rc, ColourAllocated fore, ColourAllocated back);
-  void FillRectangle(PRectangle rc, ColourAllocated back);
+  void Polygon(Scintilla::Point *pts, int npts, ColourDesired fore, ColourDesired back);
+  void RectangleDraw(PRectangle rc, ColourDesired fore, ColourDesired back);
+  void FillRectangle(PRectangle rc, ColourDesired back);
   void FillRectangle(PRectangle rc, Surface &surfacePattern);
-  void RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back);
-  void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
-                     ColourAllocated outline, int alphaOutline, int flags);
-  void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+  void RoundedRectangle(PRectangle rc, ColourDesired fore, ColourDesired back);
+  void AlphaRectangle(PRectangle rc, int cornerSize, ColourDesired fill, int alphaFill,
+                     ColourDesired outline, int alphaOutline, int flags);
+  void DrawRGBAImage(PRectangle rc, int width, int height, const unsigned char *pixelsImage);
+  void Ellipse(PRectangle rc, ColourDesired fore, ColourDesired back);
   void Copy(PRectangle rc, Scintilla::Point from, Surface &surfaceSource);
-  void DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore,
-                     ColourAllocated back);
-  void DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore, 
-                      ColourAllocated back);
-  void DrawTextTransparent(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore);
-  void MeasureWidths(Font &font_, const char *s, int len, int *positions);
-  int WidthText(Font &font_, const char *s, int len);
-  int WidthChar(Font &font_, char ch);
-  int Ascent(Font &font_);
-  int Descent(Font &font_);
-  int InternalLeading(Font &font_);
-  int ExternalLeading(Font &font_);
-  int Height(Font &font_);
-  int AverageCharWidth(Font &font_);
+  void DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len, ColourDesired fore,
+                     ColourDesired back);
+  void DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len, ColourDesired fore, 
+                      ColourDesired back);
+  void DrawTextTransparent(PRectangle rc, Font &font_, XYPOSITION ybase, const char *s, int len, ColourDesired fore);
+  void MeasureWidths(Font &font_, const char *s, int len, XYPOSITION *positions);
+  XYPOSITION WidthText(Font &font_, const char *s, int len);
+  XYPOSITION WidthChar(Font &font_, char ch);
+  XYPOSITION Ascent(Font &font_);
+  XYPOSITION Descent(Font &font_);
+  XYPOSITION InternalLeading(Font &font_);
+  XYPOSITION ExternalLeading(Font &font_);
+  XYPOSITION Height(Font &font_);
+  XYPOSITION AverageCharWidth(Font &font_);
 
-  int SetPalette(Scintilla::Palette *pal, bool inBackGround);
   void SetClip(PRectangle rc);
   void FlushCachedState();
 
   void SetUnicodeMode(bool unicodeMode_);
-  void SetDBCSMode(int codePage);
+  void SetDBCSMode(int codePage_);
 }; // SurfaceImpl class
   
 } // Scintilla namespace
