@@ -84,6 +84,11 @@ cd ..
 
 cd ScintillaEditPy
 python sepbuild.py
+cd ../../test
+python simpleTests.py
+python lexTests.py
+python performanceTests.py
+cd ../qt/ScintillaEditPy
 python sepbuild.py --clean
 cd ..
 cd ../..
@@ -132,5 +137,5 @@ cd ../..
 # ************************************************************
 # Target 7: cppcheck static checker
 # Disabled as there are false warnings and some different style choices
-#~ cppcheck --enable=all --max-configs=100 -I scintilla/src -I scintilla/include -I scintilla/lexlib -I scintilla/qt/ScintillaEditBase --template='{file}:{line}: {severity}({id}): {message}' --quiet scintilla
-#~ cppcheck --enable=all --max-configs=100 -I scite/src -I scintilla/include -I scite/lua/include --template='{file}:{line}: {severity}({id}): {message}' --quiet scite
+#~ cppcheck --enable=all --max-configs=100 --suppressions scintilla/cppcheck.suppress -I scintilla/src -I scintilla/include -I scintilla/lexlib -I scintilla/qt/ScintillaEditBase --template=gcc --quiet scintilla
+#~ cppcheck --enable=all --max-configs=100 -I scite/src -I scintilla/include -I scite/lua/include --template=gcc --quiet scite
