@@ -310,10 +310,7 @@ FilePath FilePath::NormalizePath() const {
 	strcpy(path, AsInternal());
 #ifdef WIN32
 	// Convert unix path separators to Windows
-	for (GUI::gui_char *cp = path; *cp; cp++) {
-		if (*cp == '/')
-			*cp = pathSepChar;
-	}
+	std::replace(path, path+strlen(path), L'/', pathSepChar);
 #endif
 	GUI::gui_char *absPath = new GUI::gui_char[fileName.length() + 1];
 	GUI::gui_char *cur = absPath;
@@ -678,7 +675,7 @@ std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *di
 	if (running && pi.hProcess && pi.hThread) {
 		// Wait until child process exits but time out after 5 seconds.
 		::WaitForSingleObject(pi.hProcess, 5 * 1000);
-		
+
 		DWORD bytesRead = 0;
 		DWORD bytesAvail = 0;
 		char buffer[16 * 1024];
