@@ -94,6 +94,7 @@ public:
 	sptr_t doc;
 	bool isDirty;
 	bool isReadOnly;
+	bool failedSave;
 	bool useMonoFont;
 	enum { empty, reading, readAll, open } lifeState;
 	UniMode unicodeMode;
@@ -108,7 +109,7 @@ public:
 	PropSetFile props;
 	enum FutureDo { fdNone=0, fdFinishSave=1 } futureDo;
 	Buffer() :
-			RecentFile(), doc(0), isDirty(false), isReadOnly(false), useMonoFont(false), lifeState(empty),
+			RecentFile(), doc(0), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(empty),
 			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
 			findMarks(fmNone), pFileWorker(0), futureDo(fdNone) {}
 
@@ -116,6 +117,7 @@ public:
 		RecentFile::Init();
 		isDirty = false;
 		isReadOnly = false;
+		failedSave = false;
 		useMonoFont = false;
 		lifeState = empty;
 		unicodeMode = uni8Bit;
@@ -134,10 +136,11 @@ public:
 		fileModTime = ModifiedTime();
 		fileModLastAsk = fileModTime;
 		documentModTime = fileModTime;
+		failedSave = false;
 	}
 
 	void DocumentModified();
-	bool NeedsSave(int delayBeforeSave);
+	bool NeedsSave(int delayBeforeSave) const;
 
 	void CompleteLoading();
 	void CompleteStoring();
@@ -190,7 +193,7 @@ public:
 	bool SingleBuffer() const;
 	BackgroundActivities CountBackgroundActivities() const;
 	bool SavingInBackground() const;
-	bool GetVisible(int index);
+	bool GetVisible(int index) const;
 	void SetVisible(int index, bool visible);
 	void AddFuture(int index, Buffer::FutureDo fd);
 	void FinishedFuture(int index, Buffer::FutureDo fd);
@@ -270,8 +273,8 @@ struct StyleAndWords {
 	SString words;
 	StyleAndWords() : styleNumber(0) {
 	}
-	bool IsEmpty() { return words.length() == 0; }
-	bool IsSingleChar() { return words.length() == 1; }
+	bool IsEmpty() const { return words.length() == 0; }
+	bool IsSingleChar() const { return words.length() == 1; }
 };
 
 struct CurrentWordHighlight {
@@ -392,8 +395,7 @@ protected:
 	int languageItems;
 
 	// an array of short cut items that are defined by the user in the properties file.
-	ShortcutItem *shortCutItemList; // array
-	int shortCutItems; // length of array
+	std::vector<ShortcutItem> shortCutItemList;
 
 	int codePage;
 	int characterSet;
@@ -544,7 +546,7 @@ protected:
 	void SwitchDocumentAt(int index, sptr_t pdoc);
 	int AddBuffer();
 	void UpdateBuffersCurrent();
-	bool IsBufferAvailable();
+	bool IsBufferAvailable() const;
 	bool CanMakeRoom(bool maySaveIfDirty = true);
 	void SetDocumentAt(int index, bool updateStack = true);
 	Buffer *CurrentBuffer() {
@@ -579,7 +581,7 @@ protected:
 	int GetCaretInLine();
 	void GetLine(char *text, int sizeText, int line = -1);
 	SString GetLine(int line = -1);
-	void GetRange(GUI::ScintillaWindow &win, int start, int end, char *text);
+	static void GetRange(GUI::ScintillaWindow &win, int start, int end, char *text);
 	int IsLinePreprocessorCondition(char *line);
 	bool FindMatchingPreprocessorCondition(int &curLine, int direction, int condEnd1, int condEnd2);
 	bool FindMatchingPreprocCondPosition(bool isForward, int &mppcAtCaret, int &mppcMatch);
@@ -608,7 +610,7 @@ protected:
 	void RestoreState(const Buffer &buffer, bool restoreBookmarks);
 	void Close(bool updateUI = true, bool loadingSession = false, bool makingRoomForNew = false);
 	bool IsAbsolutePath(const char *path);
-	bool Exists(const GUI::gui_char *dir, const GUI::gui_char *path, FilePath *resultPath);
+	static bool Exists(const GUI::gui_char *dir, const GUI::gui_char *path, FilePath *resultPath);
 	void DiscoverEOLSetting();
 	void DiscoverIndentSetting();
 	SString DiscoverLanguage();
@@ -637,7 +639,7 @@ protected:
 	bool Open(FilePath file, OpenFlags of = ofNone);
 	bool OpenSelected();
 	void Revert();
-	FilePath SaveName(const char *ext);
+	FilePath SaveName(const char *ext) const;
 	int SaveIfUnsure(bool forceQuestion = false);
 	int SaveIfUnsureAll(bool forceQuestion = false);
 	int SaveIfUnsureForBuilt();
@@ -674,7 +676,7 @@ protected:
 	FilePath GetLocalPropertiesFileName();
 	FilePath GetAbbrevPropertiesFileName();
 	void OpenProperties(int propsFile);
-	int GetMenuCommandAsInt(SString commandName);
+	static int GetMenuCommandAsInt(SString commandName);
 	virtual void Print(bool) {}
 	virtual void PrintSetup() {}
 	virtual void UserStripShow(const char * /* description */) {}
@@ -685,16 +687,13 @@ protected:
 	Sci_CharacterRange GetSelection();
 	SelectedRange GetSelectedRange();
 	void SetSelection(int anchor, int currentPos);
-	//	void SelectionExtend(char *sel, int len, char *notselchar);
 	SString GetCTag();
-	SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
+	static SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
 	virtual SString GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
-	SString GetLine(GUI::ScintillaWindow &win, int line);
+	static SString GetLine(GUI::ScintillaWindow &win, int line);
 	SString RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
 	        bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	SString SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
-	void FindWordAtCaret(int &start, int &end);
-	bool SelectWordAtCaret();
 	SString SelectionWord(bool stripEol = true);
 	SString SelectionFilename();
 	void SelectionIntoProperties();
@@ -729,7 +728,7 @@ protected:
 	virtual void ParamGrab() = 0;
 	virtual bool ParametersDialog(bool modal) = 0;
 	bool HandleXml(char ch);
-	SString FindOpenXmlTag(const char sel[], int nSize);
+	static SString FindOpenXmlTag(const char sel[], int nSize);
 	void GoMatchingBrace(bool select);
 	void GoMatchingPreprocCond(int direction, bool select);
 	virtual void FindReplace(bool replace) = 0;
@@ -792,7 +791,7 @@ protected:
 	void FoldAll();
 	void ToggleFoldRecursive(int line, int level);
 	void EnsureAllChildrenVisible(int line, int level);
-	void EnsureRangeVisible(GUI::ScintillaWindow &win, int posStart, int posEnd, bool enforcePolicy = true);
+	static void EnsureRangeVisible(GUI::ScintillaWindow &win, int posStart, int posEnd, bool enforcePolicy = true);
 	void GotoLineEnsureVisible(int line);
 	bool MarginClick(int position, int modifiers);
 	void NewLineInOutput();
@@ -827,7 +826,6 @@ protected:
 
 	void DeleteFileStackMenu();
 	void SetFileStackMenu();
-	void DropFileStackTop();
 	bool AddFileToBuffer(const BufferState &bufferState);
 	void AddFileToStack(FilePath file, SelectedRange selection, int scrollPos);
 	void RemoveFileFromStack(FilePath file);
@@ -840,8 +838,8 @@ protected:
 	void RemoveToolsMenu();
 	void SetMenuItemLocalised(int menuNumber, int position, int itemID,
 	        const char *text, const char *mnemonic);
+	bool ToolIsImmediate(int item);
 	void SetToolsMenu();
-	JobSubsystem SubsystemType(char c);
 	JobSubsystem SubsystemType(const char *cmd, int item = -1);
 	void ToolsMenu(int item);
 
@@ -861,7 +859,7 @@ protected:
 	void SetOverrideLanguage(int cmdID);
 	StyleAndWords GetStyleAndWords(const char *base);
 	SString ExtensionFileName();
-	const char *GetNextPropItem(const char *pStart, char *pPropItem, int maxLen);
+	static const char *GetNextPropItem(const char *pStart, char *pPropItem, int maxLen);
 	void ForwardPropertyToEditor(const char *key);
 	void DefineMarker(int marker, int markerType, Colour fore, Colour back, Colour backSelected);
 	void ReadAPI(const SString &fileNameForExtension);
@@ -966,7 +964,6 @@ private:
 #endif
 
 int ControlIDOfCommand(unsigned long);
-std::vector<GUI::gui_string> ListFromString(const GUI::gui_string &args);
 long ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
 
