@@ -442,12 +442,15 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 				buffers.RemoveInvisible(iBuffer);
 			}
 			if (iBuffer == buffers.Current()) {
-				wEditor.Call(SCI_SETSAVEPOINT);
 				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+				if (pathSaved.SameNameAs(CurrentBuffer()->AsInternal())) {
+					wEditor.Call(SCI_SETSAVEPOINT);
+				}
 				if (extender)
 					extender->OnSave(buffers.buffers[iBuffer].AsUTF8().c_str());
 			} else {
 				buffers.buffers[iBuffer].isDirty = false;
+				buffers.buffers[iBuffer].failedSave = false;
 				// Need to make writable and set save point when next receive focus.
 				buffers.AddFuture(iBuffer, Buffer::fdFinishSave);
 				SetBuffersMenu();
@@ -542,6 +545,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 	if (buffers.size == buffers.length) {
 		AddFileToStack(filePath, GetSelectedRange(), GetCurrentScrollPosition());
 		ClearDocument();
+		CurrentBuffer()->lifeState = Buffer::open;
 		if (extender)
 			extender->InitBuffer(buffers.Current());
 	} else {
@@ -776,7 +780,7 @@ void SciTEBase::Activate(bool activeApp) {
 	}
 }
 
-FilePath SciTEBase::SaveName(const char *ext) {
+FilePath SciTEBase::SaveName(const char *ext) const {
 	GUI::gui_string savePath = filePath.AsInternal();
 	if (ext) {
 		int dot = static_cast<int>(savePath.length() - 1);
@@ -958,12 +962,12 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 			if (!(sf & sfSynchronous)) {
 				wEditor.Call(SCI_SETREADONLY, 1);
 				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CallReturnPointer(SCI_GETCHARACTERPOINTER));
-				CurrentBuffer()->pFileWorker = new FileStorer(this, documentBytes, filePath, lengthDoc, fp, CurrentBuffer()->unicodeMode, (sf & sfProgressVisible));
+				CurrentBuffer()->pFileWorker = new FileStorer(this, documentBytes, saveName, lengthDoc, fp, CurrentBuffer()->unicodeMode, (sf & sfProgressVisible));
 				CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
 				if (PerformOnNewThread(CurrentBuffer()->pFileWorker)) {
 					retVal = true;
 				} else {
-					GUI::gui_string msg = LocaliseMessage("Failed to save file '^0' as thread could not be started.", filePath.AsInternal());
+					GUI::gui_string msg = LocaliseMessage("Failed to save file '^0' as thread could not be started.", saveName.AsInternal());
 					WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 				}
 			} else {
@@ -994,7 +998,7 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 		}
 	}
 
-	if (retVal && extender) {
+	if (retVal && extender && (sf & sfSynchronous)) {
 		extender->OnSave(saveName.AsUTF8().c_str());
 	}
 	UpdateStatusBar(true);
@@ -1063,6 +1067,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 				}
 			}
 		} else {
+			CurrentBuffer()->failedSave = true;
 			msg = LocaliseMessage(
 			            "Could not save file '^0'. Save under a different name?", filePath.AsInternal());
 			decision = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
@@ -1230,7 +1235,7 @@ public:
 		}
 		fp = NULL;
 	}
-	bool Exhausted() {
+	bool Exhausted() const {
 		return exhausted;
 	}
 	int NextByte() {
@@ -1303,7 +1308,7 @@ public:
 	int LineNumber() const {
 		return lineNum;
 	}
-	const char *Original() {
+	const char *Original() const {
 		return lineToShow;
 	}
 	bool BufferContainsNull() {
