@@ -6,6 +6,7 @@
 // The License.txt file describes the conditions under which this software may be distributed.
 
 #include <string.h>
+#include <stdio.h>
 
 #include <string>
 #include <vector>
@@ -38,6 +39,12 @@ int Substitute(GUI::gui_string &s, const GUI::gui_string &sFind, const GUI::gui_
 	return c;
 }
 
+std::string StdStringFromInteger(int i) {
+	char number[32];
+	sprintf(number, "%0d", i);
+	return std::string(number);
+}
+
 /**
  * Convert a string into C string literal form using \a, \b, \f, \n, \r, \t, \v, and \ooo.
  * The return value is a newly allocated character array containing the result.
