@@ -23,7 +23,8 @@ void SciTEWin::SetFileProperties(
 		FILETIME lft;
 		::FileTimeToLocalFileTime(&ft, &lft);
 		SYSTEMTIME st;
-		::FileTimeToSystemTime(&lft, &st);
+		if (::FileTimeToSystemTime(&lft, &st) == 0)
+			memset(&st, 0, sizeof(st));
 		::GetTimeFormatA(LOCALE_USER_DEFAULT,
 		                0, &st,
 		                NULL, temp, TEMP_LEN);
@@ -178,7 +179,6 @@ void SciTEWin::Notify(SCNotification *notification) {
 	case TTN_GETDISPINFO:
 		// Ask for tooltip text
 		{
-			static GUI::gui_char ttt[MAX_PATH*2 + 1];
 			const GUI::gui_char *ttext = 0;
 			NMTTDISPINFOW *pDispInfo = (NMTTDISPINFOW *)notification;
 			// Toolbar tooltips
@@ -252,16 +252,16 @@ void SciTEWin::Notify(SCNotification *notification) {
 							path.insert(amp, GUI_TEXT("&"));
 							amp += 2;
 						}
-						wcscpy(ttt, path.c_str());
-						pDispInfo->lpszText = const_cast<GUI::gui_char *>(ttt);
+						StringCopy(tooltipText, path.c_str());
+						pDispInfo->lpszText = tooltipText;
 					}
 				}
 				break;
 			}
 			if (ttext) {
 				GUI::gui_string localised = localiser.Text(GUI::UTF8FromString(ttext).c_str());
-				wcscpy(ttt, localised.c_str());
-				pDispInfo->lpszText = ttt;
+				StringCopy(tooltipText, localised.c_str());
+				pDispInfo->lpszText = tooltipText;
 			}
 			break;
 		}
@@ -457,7 +457,7 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 		keycode = SciTEKeys::ParseKeyCode(GUI::UTF8FromString(mnemonic).c_str());
 		if (keycode) {
 			sTextMnemonic += GUI_TEXT("\t");
-			sTextMnemonic += LocaliseAccelerator(mnemonic, itemID);
+			sTextMnemonic += mnemonic;
 		}
 		// the keycode could be used to make a custom accelerator table
 		// but for now, the menu's item data is used instead for command
@@ -529,104 +529,6 @@ void SciTEWin::CheckMenus() {
 	                   CurrentBuffer()->unicodeMode + IDM_ENCODING_DEFAULT, 0);
 }
 
-void SciTEWin::MakeAccelerator(SString sAccelerator, ACCEL &Accel) {
-	SString s = sAccelerator;
-
-	if (s.contains("null")) {
-		Accel.key = 0;
-		return ;
-	}
-
-	if (s.contains("Ctrl+")) {
-		Accel.fVirt |= FCONTROL;
-		s.remove("Ctrl+");
-	}
-	if (s.contains("Shift+")) {
-		Accel.fVirt |= FSHIFT;
-		s.remove("Shift+");
-	}
-	if (s.contains("Alt+")) {
-		Accel.fVirt |= FALT;
-		s.remove("Alt+");
-	}
-	if (s.length() == 1) {
-		Accel.key = s[0];
-		Accel.fVirt |= FVIRTKEY;
-	} else if ((s.length() > 1) && (s[0] == 'F') && (isdigit(s[1]))) {
-		s.remove("F");
-		int keyNum = s.value();
-		Accel.key = static_cast<WORD>(keyNum + VK_F1 - 1);
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Del")) {
-		Accel.key = VK_DELETE;
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Space")) {
-		Accel.key = VK_SPACE;
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Enter")) {
-		Accel.key = VK_RETURN;
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Back")) {
-		Accel.key = VK_BACK;
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Tab")) {
-		Accel.key = VK_TAB;
-		Accel.fVirt |= FVIRTKEY;
-	} else if (s.contains("Num")) {
-		Accel.fVirt |= FVIRTKEY;
-		s.remove("Num");
-		if (isdigit(s[0])) {
-			int keyNum = s.value();
-			Accel.key = static_cast<WORD>(keyNum + VK_NUMPAD0);
-		} else {
-			switch (s[0]) {
-			case '*':
-				Accel.key = VK_MULTIPLY;
-				break;
-			case '+':
-				Accel.key = VK_ADD;
-				break;
-			case '-':
-				Accel.key = VK_SUBTRACT;
-				break;
-			case '/':
-				Accel.key = VK_DIVIDE;
-				break;
-			default:
-				Accel.key = 0;
-				break;
-			}
-		}
-	}
-}
-
-//SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int cmd) {
-GUI::gui_string SciTEWin::LocaliseAccelerator(const GUI::gui_char *pAccelerator, int) {
-#ifdef LOCALISE_ACCELERATORS_WORKED
-	SString translation = localiser.Text(pAccelerator, true);
-	int AccelCount = ::CopyAcceleratorTable(hAccTable, NULL, 0);
-	ACCEL *AccelTable = new ACCEL[AccelCount];
-	::CopyAcceleratorTable(hAccTable, AccelTable, AccelCount);
-	for (int i = 0; i < AccelCount; i++) {
-		if (AccelTable[i].cmd == cmd) {
-			MakeAccelerator(translation, AccelTable[i]);
-		}
-	}
-
-	::DestroyAcceleratorTable(hAccTable);
-	hAccTable = ::CreateAcceleratorTable(AccelTable, AccelCount);
-	delete []AccelTable;
-
-	if (translation.contains("null")) {
-		translation.clear();
-	}
-
-	return translation;
-#else
-	return pAccelerator;
-#endif
-}
-
 void SciTEWin::LocaliseMenu(HMENU hmenu) {
 	for (int i = 0; i <= ::GetMenuItemCount(hmenu); i++) {
 		GUI::gui_char buff[200];
@@ -658,7 +560,7 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 					if (text.length()) {
 						if (accel != GUI_TEXT("")) {
 							text += GUI_TEXT("\t");
-							text += LocaliseAccelerator(accel.c_str(), mii.wID);
+							text += accel;
 						}
 						mii.dwTypeData = const_cast<GUI::gui_char *>(text.c_str());
 						::SetMenuItemInfoW(hmenu, i, TRUE, &mii);
@@ -674,22 +576,6 @@ void SciTEWin::LocaliseMenus() {
 	::DrawMenuBar(MainHWND());
 }
 
-void SciTEWin::LocaliseAccelerators() {
-	LocaliseAccelerator(GUI_TEXT("Alt+1"), IDM_BUFFER + 0);
-	LocaliseAccelerator(GUI_TEXT("Alt+2"), IDM_BUFFER + 1);
-	LocaliseAccelerator(GUI_TEXT("Alt+3"), IDM_BUFFER + 2);
-	LocaliseAccelerator(GUI_TEXT("Alt+4"), IDM_BUFFER + 3);
-	LocaliseAccelerator(GUI_TEXT("Alt+5"), IDM_BUFFER + 4);
-	LocaliseAccelerator(GUI_TEXT("Alt+6"), IDM_BUFFER + 5);
-	LocaliseAccelerator(GUI_TEXT("Alt+7"), IDM_BUFFER + 6);
-	LocaliseAccelerator(GUI_TEXT("Alt+8"), IDM_BUFFER + 7);
-	LocaliseAccelerator(GUI_TEXT("Alt+9"), IDM_BUFFER + 8);
-	LocaliseAccelerator(GUI_TEXT("Alt+0"), IDM_BUFFER + 9);
-
-	// todo read keymap from cfg
-	// AssignKey('Y', SCMOD_CTRL, SCI_LINECUT);
-}
-
 void SciTEWin::LocaliseControl(HWND w) {
 	char wtext[200];
 	if (::GetWindowTextA(w, wtext, sizeof(wtext))) {
@@ -1029,7 +915,8 @@ void SciTEWin::Creation() {
 	InitCommonControlsEx(&icce);
 
 	WNDCLASS wndClass = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-	GetClassInfo(NULL, WC_TABCONTROL, &wndClass);
+	if (::GetClassInfo(NULL, WC_TABCONTROL, &wndClass) == 0)
+		exit(FALSE);
 	stDefaultTabProc = wndClass.lpfnWndProc;
 	wndClass.lpfnWndProc = TabWndProc;
 	wndClass.style = wndClass.style | CS_DBLCLKS;
@@ -1056,7 +943,8 @@ void SciTEWin::Creation() {
 
 	LOGFONT lfIconTitle;
 	ZeroMemory(&lfIconTitle, sizeof(lfIconTitle));
-	::SystemParametersInfo(SPI_GETICONTITLELOGFONT,sizeof(lfIconTitle),&lfIconTitle,FALSE);
+	if (::SystemParametersInfo(SPI_GETICONTITLELOGFONT,sizeof(lfIconTitle),&lfIconTitle,FALSE) == 0)
+		exit(FALSE);
 	fontTabs = ::CreateFontIndirect(&lfIconTitle);
 	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()),
 	              WM_SETFONT,
